/*
 * CORBA_quo.java
 *
 * Copyright (c) 2003 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is licensed under the terms of the GNU Public License.  
 * See the file "license.terms" for restrictions on redistribution 
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 */

import java.io.*;

import imagerec.corba.CORBA;

import org.omg.CORBA.ORB;
import org.omg.CORBA.Policy;
import org.omg.PortableServer.POA;
import org.omg.PortableServer.POAHelper;
import org.omg.PortableServer.Servant;
import org.omg.PortableServer.ThreadPolicy;
import org.omg.PortableServer.ThreadPolicyValue;

import org.omg.CosNaming.NamingContextExt;
import org.omg.CosNaming.NamingContextExtHelper;
import org.omg.CosNaming.NameComponent;

import com.bbn.quo.*;
import com.bbn.quo.corba.*;

import Broker.Manager;
import Broker.NamedValue;
import Broker.TaskFactory;
import Broker.RealTimeAdvocate;

import unix.Process;

/**
 * A CORBA CommunicationsModel for the ATR that includes support for QuO.
 */
public class CORBA_quo
    extends CORBA
{
    static {
	System.loadLibrary("JavaUnixUtils"); // XXX grr stupid!
    }

    private static String readIORFile(String filename)
	throws Exception
    {
	byte retval[];

	retval = new byte[(int)new File(filename).length()];
	new FileInputStream(filename).read(retval);
	return new String(retval);
    }

    private String args[];
    private ORB orb;
    
    public CORBA_quo(String args[])
    {
	super(args);
	
	this.args = args;
	this.orb = ORB.init(args, null);
    }
    
    protected void runServer(String name, final Servant servant)
	throws Exception
    {
	POA root_poa;

	root_poa = POAHelper.narrow(orb.resolve_initial_references("RootPOA"));
	root_poa.the_POAManager().activate();

	ThreadPolicy thread =
	    root_poa.create_thread_policy(ThreadPolicyValue.
					  SINGLE_THREAD_MODEL);

	POA poa = root_poa.create_POA("SingleThread",
				      root_poa.the_POAManager(),
				      new Policy[] {
					  thread
				      });
	
	poa.activate_object(servant);
	
	NamingContextExt namingContext = namingContext(orb);
	namingContext.rebind(namingContext.to_name("real_" + name),
			     poa.servant_to_reference(servant));
	
	RealTimeAdvocate rta;
	Manager manager;
	TaskFactory tf;
	QuoKernel qk;

	qk = QuoKernelHelper.
	    narrow(this.orb.string_to_object(readIORFile("quoKernel.ior")));
	
	tf = Broker.TaskFactoryHelper.
	    narrow(this.orb.string_to_object(readIORFile("max_task_factory.ior")));

	manager = Broker.ManagerHelper.
	    narrow(this.orb.string_to_object(readIORFile("manager.ior")));

	org.omg.CORBA.Any name_any = orb.create_any();
	name_any.insert_string(name);
	
	org.omg.CORBA.Any pid_any = orb.create_any();
	pid_any.insert_long(Process.getProcessID());

	org.omg.CORBA.Any res_log_any = orb.create_any();
	res_log_any.insert_string("/tmp/atr-res.out");

	org.omg.CORBA.Any cpu_log_any = orb.create_any();
	cpu_log_any.insert_string("/tmp/atr-cpu.out");

	org.omg.CORBA.Any advice_log_any = orb.create_any();
	advice_log_any.insert_string("/tmp/atr-advice.out");

	org.omg.CORBA.Any complete_log_any = orb.create_any();
	complete_log_any.insert_string("/tmp/atr-complete.out");

	System.out.println("create task");
	rta = Broker.RealTimeAdvocateHelper.
	    narrow(tf.CreateTask(new NamedValue[] {
		new NamedValue("name", name_any),
		new NamedValue("pid", pid_any),
		new NamedValue("res log", res_log_any),
		new NamedValue("cpu log", cpu_log_any),
		new NamedValue("advice log", advice_log_any),
		new NamedValue("complete log", complete_log_any),
	    }));

	System.out.println("begin schedule");
	org.omg.CORBA.Any period_any = orb.create_any();
	period_any.insert_ulong(500000);
	rta.BeginCPUScheduling(manager, new NamedValue[] {
	    new NamedValue("period", period_any),
	});
	
	ATRWrapper aw = new ATRWrapper(rta, 500000, 500000);

	aw.initSysconds(qk);
	aw.initCallbacks();
	aw.linkContract(qk);
	System.out.println("link");
	aw.linkRemoteObject(FrameManip.ProcessorHelper.
			    narrow(servant._this_object()));
	
	poa.activate_object(aw);
	
	System.out.println("run");
	
	namingContext.rebind(namingContext.to_name(name),
			     poa.servant_to_reference(aw));
	
	orb.run();
    }
}
