/*
 * ManagerImpl.hh
 *
 * Copyright (c) 2003 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is licensed under the terms of the GNU Public License.  
 * See the file "license.terms" for restrictions on redistribution 
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 */

/**
 * @file ManagerImpl.hh
 *
 * Header file for the default implementation of the Broker::Manager interface.
 */

#ifndef _manager_impl_hh
#define _manager_impl_hh

#include "BrokerS.h"

#include <listNode.h>

/**
 * A default implementation of the Broker::Manager interface.
 */
class ManagerImpl : public virtual POA_Broker::Manager
{

public:

    /**
     * Construct a ManagerImpl.
     *
     * @param name The name of this manager.
     */
    ManagerImpl(const char *name);

    /**
     * Destruct a ManagerImpl.
     */
    virtual ~ManagerImpl();

    /** @copydoc Broker::Manager::CurrentPolicy */
    Broker::Policy_ptr CurrentPolicy(void)
	throw (CORBA::SystemException);
    
    /** @copydoc Broker::Manager::CurrentPolicy */
    void CurrentPolicy(Broker::Policy_ptr policy)
	throw (CORBA::SystemException);

    /** @copydoc Broker::Manager::AddTask */
    void AddTask(Broker::Task_ptr task)
	throw (CORBA::SystemException);

    /** @copydoc Broker::Manager::RemoveTask */
    void RemoveTask(Broker::Task_ptr task)
	throw (CORBA::SystemException);

    /** @copydoc Broker::Manager::ChangeTaskCPU */
    void ChangeTaskCPU(Broker::RealTimeTask_ptr task,
		       CORBA::ULong ct,
		       CORBA::ULong status,
		       CORBA::ULong advice)
	throw (CORBA::SystemException, Broker::InvalidState);
    
private:

    /**
     * Get the index of the given task in the mi_Tasks sequence.
     *
     * @param task The task object to look up.
     * @return The index of the given task in the mi_Tasks sequence or -1 if
     *         the task is not in the sequence.
     */
    int TaskIndex(Broker::Task_ptr task);
    
    /**
     * The active policy object or nil.
     */
    Broker::Policy_var mi_CurrentPolicy;

    /**
     * The list of tasks currently being managed.  This list will be passed
     * to policies when they are Broker::Policy::Activate()'d.
     */
    Broker::TaskList mi_Tasks;
    
};

#endif
