/*
 * TaskFactory_T.hh
 *
 * Copyright (c) 2003 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is licensed under the terms of the GNU Public License.  
 * See the file "license.terms" for restrictions on redistribution 
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 */

/**
 * @file TaskFactory_T.hh
 *
 * Header file that provides a template for creating task factories.
 */

#ifndef _task_factory_t_hh
#define _task_factory_t_hh

#include "BrokerC.h"
#include "BrokerS.h"

/**
 * Template for task factories that just construct a task with the
 * Broker::TaskParameters passed to Broker::TaskFactory::CreateTask.
 */
template <class T> class TaskFactoryTemplate :
    public POA_Broker::TaskFactory
{

public:

    /**
     * Empty constructor.
     */
    TaskFactoryTemplate(const char *name)
    {
    };

    /**
     * Empty destructor.
     */
    virtual ~TaskFactoryTemplate()
    {
    };

    /** @copydoc Broker::TaskFactory::CreateTask */
    Broker::Task_ptr CreateTask(const Broker::TaskParameters &tp)
	throw (CORBA::SystemException,
	       Broker::DuplicateTaskParameter,
	       Broker::InvalidTaskParameter,
	       Broker::MissingTaskParameter)
    {
	T *retval;
	
	retval = new T(tp);
	return( retval->_this() );
    }
    
};

#endif
