/*
 * ManagerImpl.cc
 *
 * Copyright (c) 2003 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is licensed under the terms of the GNU Public License.  
 * See the file "license.terms" for restrictions on redistribution 
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 */

/**
 * @file ManagerImpl.cc
 *
 * Implementation of the ManagerImpl class.
 */

#include "config.h"

#include <assert_pp.h>

#include "ManagerImpl.hh"

ManagerImpl::ManagerImpl(const char *name)
{
}

ManagerImpl::~ManagerImpl()
{
}

int ManagerImpl::TaskIndex(Broker::Task_ptr task)
{
    unsigned int lpc;
    int retval = -1;

    require(!CORBA::is_nil(task));
    
    for( lpc = 0; (lpc < this->mi_Tasks.length()) && (retval == -1); lpc++ )
    {
	if( this->mi_Tasks[lpc]->_is_equivalent(task) )
	{
	    retval = lpc;
	}
    }
    return( retval );
}

Broker::Policy_ptr ManagerImpl::CurrentPolicy(void)
    throw (CORBA::SystemException)
{
    Broker::Policy_var retval;

    retval = this->mi_CurrentPolicy;
    return( retval._retn() );
}

void ManagerImpl::CurrentPolicy(Broker::Policy_ptr policy)
    throw (CORBA::SystemException)
{
    if( !CORBA::is_nil(this->mi_CurrentPolicy.in()) )
    {
	this->mi_CurrentPolicy->Deactivate();
    }
    this->mi_CurrentPolicy = Broker::Policy::_duplicate(policy);
    if( !CORBA::is_nil(this->mi_CurrentPolicy.in()) )
    {
	this->mi_CurrentPolicy->Activate(this->mi_Tasks);
    }
}

void ManagerImpl::AddTask(Broker::Task_ptr task)
    throw (CORBA::SystemException)
{
    int slot;

    if( CORBA::is_nil(task) )
    {
	throw CORBA::BAD_PARAM();
    }

    if( this->TaskIndex(task) != -1 )
    {
	throw CORBA::BAD_PARAM();
    }
    
    slot = this->mi_Tasks.length();
    this->mi_Tasks.length(slot + 1);
    this->mi_Tasks[slot] = Broker::Task::_duplicate(task);

    if( !CORBA::is_nil(this->mi_CurrentPolicy.in()) )
    {
	this->mi_CurrentPolicy->AddTask(task);
    }
}

void ManagerImpl::RemoveTask(Broker::Task_ptr task)
    throw (CORBA::SystemException)
{
    unsigned int lpc;
    int slot;
    
    if( CORBA::is_nil(task) )
    {
	throw CORBA::BAD_PARAM();
    }

    /* Find the task, ... */
    if( (slot = this->TaskIndex(task)) == -1 )
    {
	throw CORBA::BAD_PARAM();
    }

    /* fill the new gap, and ... */
    for( lpc = (unsigned int)slot; lpc < (this->mi_Tasks.length() - 1); lpc++ )
    {
	this->mi_Tasks[lpc] = this->mi_Tasks[lpc + 1];
    }
    this->mi_Tasks.length(this->mi_Tasks.length() - 1);

    /* tell the policy. */
    if( !CORBA::is_nil(this->mi_CurrentPolicy.in()) )
    {
	this->mi_CurrentPolicy->RemoveTask(task);
    }
}

void ManagerImpl::ChangeTaskCPU(Broker::RealTimeTask_ptr task,
				CORBA::ULong ct,
				CORBA::ULong status,
				CORBA::ULong advice)
    throw (CORBA::SystemException, Broker::InvalidState)
{
    if( CORBA::is_nil(task) )
    {
	throw CORBA::BAD_PARAM();
    }

    if( !CORBA::is_nil(this->mi_CurrentPolicy.in()) )
    {
	/* Have a policy, just pass it on. */
	this->mi_CurrentPolicy->ChangeTaskCPU(task, ct, status, advice);
    }
    else
    {
	/* No policy, just set the time and return. */
	try
	{
	    task->SetComputeTime(advice);
	}
	catch(...)
	{
	    /* XXX */
	}
    }
}
