/*
 * BasicTaskQosketImpl.hh
 *
 * Copyright (c) 2003 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is licensed under the terms of the GNU Public License.  
 * See the file "license.terms" for restrictions on redistribution 
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 */

/**
 * @file BasicTaskQosketImpl.hh
 *
 * Header file for the BasicTaskQosketImpl class.
 */

#ifndef _basic_task_qosket_impl_hh
#define _basic_task_qosket_impl_hh

#include "BrokerC.h"
#include "BasicTaskQosketSkel.h"

#include "time_util.h" // XXX

/**
 * A subclass of the Qosket::Basic::BasicTaskQosketSkel class generated from
 * the BasicTaskContract.  The implementation is pretty straightforward, it
 * gets its system conditions from the quo::QuoKernel and creates the callback
 * itself.
 *
 * @sa Qosket::Basic::BasicTaskQosketImpl
 * @sa Qosket::Basic::BasicTaskCallback
 * @sa BasicTaskCallbackImpl
 * @sa StubBasicTaskCallback
 * @sa BasicTaskContract.cdl
 */
class BasicTaskQosketImpl :
    public virtual Qosket::Basic::BasicTaskQosketSkel
{

public:

    /**
     * Initialize the object with the required QuO system conditions by calling
     * bindSysCond on the given kernel object.
     *
     * @param kernel The quo::QuoKernel that will be the source of any system
     *               conditions.
     * @exception CORBA::SystemException descendant if something goes wrong
     *            while interacting with the kernel.
     */
    virtual void initSysconds(const quo::QuoKernel_ptr &kernel)
	throw (CORBA::SystemException);

    /**
     * Creates a new BasicTaskCallbackImpl object and installs it in the
     * myCallback field.
     *
     * @param orb Unused.
     */
    virtual void initCallbacks(const CORBA::ORB_ptr &orb)
	throw (CORBA::SystemException);

    /** @copydoc Qosket::Basic::BasicTaskQosket::DeadlinesMissed */
    virtual quo::ValueSC_ptr DeadlinesMissed(void)
	throw (CORBA::SystemException);
    
    virtual void DeadlinesMissed(quo::ValueSC_ptr)
	throw (CORBA::SystemException);

    /** @copydoc Qosket::Basic::BasicTaskQosket::LastStatus */
    virtual quo::ValueSC_ptr LastStatus(void)
	throw (CORBA::SystemException);

    virtual void LastStatus(quo::ValueSC_ptr)
	throw (CORBA::SystemException);

    /** @copydoc Qosket::Basic::BasicTaskQosket::Callback */
    virtual quo::Callback_ptr Callback(void)
	throw (CORBA::SystemException);
    
    virtual void Callback(quo::Callback_ptr)
	throw (CORBA::SystemException);

    /** @copydoc Qosket::Basic::BasicTaskQosket::RUsage */
    virtual CORBA::ULongLong RUsage(void)
	throw (CORBA::SystemException);
    
    virtual void RUsage(CORBA::ULongLong)
	throw (CORBA::SystemException);

};

#endif
