/*
 * rk_ResourceSet.c
 *
 * Copyright (c) 2003 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is licensed under the terms of the GNU Public License.  
 * See the file "license.terms" for restrictions on redistribution 
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 */

#include "config.h"

#include <stdio.h>
#include <string.h>
#include <unistd.h>

#include <sys/types.h>

#include <rk/rk.h>

#include "rk_ResourceSet.h"

#define UNUSED_ARG(a) do {/* null */} while (&a == 0)

JNIEXPORT jlong JNICALL
Java_rk_ResourceSet_getUsage(JNIEnv *env, jclass java_class)
{
#define MAX_PID_COUNT 128
    static pid_t procs[MAX_PID_COUNT];

    rk_resource_set_t rs;
    jlong retval = 0;
    
    UNUSED_ARG(env);
    UNUSED_ARG(java_class);

    if( (rs = rk_proc_get_rset(getpid())) != NULL_RESOURCE_SET )
    {
	int lpc, count;
	
	count = rk_resource_set_get_proclist(rs, procs, MAX_PID_COUNT);
	for( lpc = 0; lpc < count; lpc++ )
	{
	    static char filename[1024];

	    char *unused_string = filename;
	    unsigned long unused_ulong;
	    long unused_long;
	    char unused_char;
	    int unused_int;
	    FILE *file;
	    
	    snprintf(filename,
		     sizeof(filename),
		     "/proc/%d/stat",
		     procs[lpc]);
	    if( (file = fopen(filename, "r")) != NULL )
	    {
		struct timeval utime, stime;

		memset(&utime, 0, sizeof(utime));
		memset(&stime, 0, sizeof(stime));
		fscanf(file,
		       "%d %s %c %d %d %d %d %d %lu %lu %lu %lu %lu %lu %lu %ld %ld %ld %ld %ld %ld %lu %lu %ld %lu %lu %lu %lu %lu %lu %lu %lu %lu %lu %lu %lu %lu %d %d %ld %ld %ld %ld",
		       &unused_int,
		       unused_string,
		       &unused_char,
		       &unused_int,
		       &unused_int,
		       &unused_int,
		       &unused_int,
		       &unused_int,
		       &unused_ulong,
		       &unused_ulong,
		       &unused_ulong,
		       &unused_ulong,
		       &unused_ulong,
		       &unused_ulong,
		       &unused_ulong,
		       &unused_long,
		       &unused_long,
		       &unused_long,
		       &unused_long,
		       &unused_long,
		       &unused_long,
		       &unused_ulong,
		       &unused_ulong,
		       &unused_long,
		       &unused_ulong,
		       &unused_ulong,
		       &unused_ulong,
		       &unused_ulong,
		       &unused_ulong,
		       &unused_ulong,
		       &unused_ulong,
		       &unused_ulong,
		       &unused_ulong,
		       &unused_ulong,
		       &unused_ulong,
		       &unused_ulong,
		       &unused_ulong,
		       &unused_int,
		       &unused_int,
		       &utime.tv_sec,
		       &utime.tv_usec,
		       &stime.tv_sec,
		       &stime.tv_usec);
		retval += utime.tv_sec * 1000000;
		retval += utime.tv_usec;
		retval += stime.tv_sec * 1000000;
		retval += stime.tv_usec;
		fclose(file);
	    }
	}
    }
    return( retval );
}
