/*
 * ExactTaskAdvocate.hh
 *
 * Copyright (c) 2003 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is licensed under the terms of the GNU Public License.  
 * See the file "license.terms" for restrictions on redistribution 
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 */

/**
 * @file ExactTaskAdvocate.hh
 *
 * Header file for a task advocate class that suggests the task get exactly the
 * amount of compute time as needed by the last period.
 */

#ifndef _exact_task_advocate_hh
#define _exact_task_advocate_hh

#include <iostream>

#include "RKTask.hh"

/**
 * A Broker::RealTimeAdvocate that suggests the task gets the exact amount of
 * compute time as needed by the last period.
 */
class ExactTaskAdvocate : public virtual RKTask,
			  public virtual POA_Broker::RealTimeAdvocate
{

public:

    /** @copydoc Broker::TaskFactory::CreateTask */
    ExactTaskAdvocate(const Broker::TaskParameters &tp)
	throw (CORBA::SystemException,
	       Broker::DuplicateTaskParameter,
	       Broker::InvalidTaskParameter,
	       Broker::MissingTaskParameter);

    virtual ~ExactTaskAdvocate();

    /**
     * Compute a new processing time advice from the given status value.  The
     * default implementation will compute the exact amount of processing time
     * required to make the deadline.
     *
     * @param status The value passed into ReportCPU.
     * @return The exact processing time required for the process to make its
     *         deadline.
     * @exception InvalidStatus if the status value is invalid.
     * @exception CORBA::BAD_INV_ORDER if the method is called without
     *            BeginCPUScheduling() being called first.
     */
    virtual CORBA::ULong ComputeAdvice(CORBA::ULong status)
	throw (CORBA::SystemException, Broker::InvalidStatus);

    /** @copydoc Broker::RealTimeAdvocate::ReportCPU */
    virtual void ReportCPU(CORBA::ULong status)
	throw (CORBA::SystemException, Broker::InvalidStatus);

protected:

    struct timeval eta_LastCPULog;

    ofstream *eta_CPULog;

    struct timeval eta_LastAdviceLog;

    ofstream *eta_AdviceLog;

    ofstream *eta_CompleteLog;

};

#endif
