/*
 * ExactTaskAdvocate_Quo.hh
 *
 * Copyright (c) 2003 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is licensed under the terms of the GNU Public License.  
 * See the file "license.terms" for restrictions on redistribution 
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 */

/**
 * @file ExactTaskAdvocate_Quo.hh
 *
 * Header file for the ExactTaskAdvocate_Quo class.
 */

#ifndef _exact_task_advocate_quo_hh
#define _exact_task_advocate_quo_hh

#include <BrokerC.h>
#include <quo/quoC.h>

#include "ExactTaskAdvocate.hh"

/**
 * An ExactTaskAdvocate that includes some support for Quo.
 *
 * XXX Make me a template.
 */
class ExactTaskAdvocate_Quo : public virtual ExactTaskAdvocate
{

public:

    /**
     * Construct an ExactTaskAdvocate_Quo object.  In addition to the
     * parameters recognized by its parents, this implementation will
     * recognize:
     *
     * @li @e LastStatus A Quo system condition that will store the last
     * reported status value.  (@e optional).
     *
     * @copydoc Broker::TaskFactory::CreateTask
     *
     * @sa RKTask::RKTask
     */
    ExactTaskAdvocate_Quo(const Broker::TaskParameters &tp)
	throw (CORBA::SystemException,
	       Broker::DuplicateTaskParameter,
	       Broker::InvalidTaskParameter,
	       Broker::MissingTaskParameter);
    
    /**
     * @copydoc Broker::RealTimeAdvocate::ReportCPU
     *
     * The implementation in this subclass will call the parent's ReportCPU and
     * then set the LastStatus system condition.
     */
    virtual void ReportCPU(CORBA::ULong status)
	throw (CORBA::SystemException, Broker::InvalidStatus);

    /**
     * @return The Quo system condition that stores the last reported status
     *         value.
     */
    quo::ValueSC_ptr GetLastStatus(void);

protected:

    /**
     * The Quo system condition that stores the last reported status value.
     */
    quo::ValueSC_var eta_LastStatus;
    
};

#endif
