/*
 * GlacialTaskAdvocate.hh
 *
 * Copyright (c) 2003 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is licensed under the terms of the GNU Public License.  
 * See the file "license.terms" for restrictions on redistribution 
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 */

/**
 * @file GlacialTaskAdvocate.hh
 *
 * Header file for the GlacialTaskAdvocate class.
 */

#ifndef _glacial_task_advocate_hh
#define _glacial_task_advocate_hh

#include "ExactTaskAdvocate.hh"

/**
 * An ExactTaskAdvocate that is slower to react to changes in the required
 * amount of CPU time.
 */
class GlacialTaskAdvocate : public ExactTaskAdvocate
{

public:
    
    /** @copydoc Broker::TaskFactory::CreateTask */
    GlacialTaskAdvocate(const Broker::TaskParameters &tp)
	throw (CORBA::SystemException,
	       Broker::DuplicateTaskParameter,
	       Broker::InvalidTaskParameter,
	       Broker::MissingTaskParameter);

    /**
     * Compute a new processing time advice from the given status value.  The
     * glacial implementation will use the advice reported by the parent class
     * and temper the change by some factor.
     *
     * @param status The value passed into ReportCPU.
     * @return A processing time with a smaller change than that reported by
     *         the parent.
     * @exception InvalidStatus if the status value is invalid.
     * @exception CORBA::BAD_INV_ORDER if the method is called without
     *            BeginCPUScheduling() being called first.
     */
    virtual CORBA::ULong ComputeAdvice(CORBA::ULong status)
	throw (CORBA::SystemException, Broker::InvalidStatus);

};

#endif
