/*
 * ExactTaskAdvocate_Quo.cc
 *
 * Copyright (c) 2003 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is licensed under the terms of the GNU Public License.  
 * See the file "license.terms" for restrictions on redistribution 
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 */

/**
 * @file ExactTaskAdvocate_Quo.cc
 *
 * Implementation of the ExactTaskAdvocate_Quo class.
 */

#include "config.h"

#include "ExactTaskFactory_Quo.hh"
#include "ExactTaskAdvocate_Quo.hh"

template TaskFactoryTemplate<ExactTaskAdvocate_Quo>;

ExactTaskAdvocate_Quo::ExactTaskAdvocate_Quo(const Broker::TaskParameters &tp)
    throw (CORBA::SystemException,
	   Broker::DuplicateTaskParameter,
	   Broker::InvalidTaskParameter,
	   Broker::MissingTaskParameter) :
    ExactTaskAdvocate(tp), RKTask(tp)
{
    unsigned int lpc;

    /* Process the arguments */
    for( lpc = 0; lpc < tp.length(); lpc++ )
    {
	if( (strcasecmp(tp[lpc].name.in(), "last_status") == 0) ||
	    (strcasecmp(tp[lpc].name.in(), "laststatus") == 0) ||
	    (strcasecmp(tp[lpc].name.in(), "last status") == 0) )
	{
	    quo::ValueSC_ptr sc;
	    
	    if( tp[lpc].value >>= sc )
	    {
		this->eta_LastStatus = quo::ValueSC::_duplicate(sc);
	    }
	    else
	    {
		throw Broker::InvalidTaskParameter("'last_status' not a "
						   "system condition",
						   tp[lpc]);
	    }
	}
    }
}

void ExactTaskAdvocate_Quo::ReportCPU(CORBA::ULong status)
    throw (CORBA::SystemException, Broker::InvalidStatus)
{
    this->ExactTaskAdvocate::ReportCPU(status);
    if( !CORBA::is_nil(this->eta_LastStatus.in()) )
    {
	this->eta_LastStatus->longValue(status);
    }
}

quo::ValueSC_ptr ExactTaskAdvocate_Quo::GetLastStatus(void)
{
    quo::ValueSC_var retval;

    retval = this->eta_LastStatus;
    return( retval._retn() );
}
