/*
 * GlacialTaskAdvocate.cc
 *
 * Copyright (c) 2003 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is licensed under the terms of the GNU Public License.  
 * See the file "license.terms" for restrictions on redistribution 
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 */

/**
 * @file GlacialTaskAdvocate.cc
 *
 * Implementation of the GlacialTaskAdvocate class.
 */

#include "config.h"

#include "GlacialTaskFactory.hh"
#include "GlacialTaskAdvocate.hh"

template TaskFactoryTemplate<GlacialTaskAdvocate>;

static void timespec_to_microsec (struct timespec time_spec,
				  CORBA::ULong &microseconds)
{
    microseconds = time_spec.tv_sec * 1000000;
    microseconds += time_spec.tv_nsec / 1000;
}

GlacialTaskAdvocate::GlacialTaskAdvocate(const Broker::TaskParameters &tp) 
    throw (CORBA::SystemException,
	   Broker::DuplicateTaskParameter,
	   Broker::InvalidTaskParameter,
	   Broker::MissingTaskParameter) :
    ExactTaskAdvocate(tp), RKTask(tp)
{
}

CORBA::ULong GlacialTaskAdvocate::ComputeAdvice(CORBA::ULong status)
    throw (CORBA::SystemException, Broker::InvalidStatus)
{
    CORBA::ULong ct, change, retval;
    
    retval = this->ExactTaskAdvocate::ComputeAdvice(status);
    timespec_to_microsec(this->rkt_CPUReserveSpec.compute_time, ct);
    if( retval > ct )
    {
	change = retval - ct;
	retval = ct + (change / 4) + 10;
    }
    else
    {
	change = ct - retval;
	retval = ct - (change / 4);
    }
    return( retval );
}
